# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["HistoryListQuarksParams"]


class HistoryListQuarksParams(TypedDict, total=False):
    lattice_id: str

    max_timestamp: int

    min_timestamp: int

    registry_identifier: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import context_list_files_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .extractors import (
    ExtractorsResource,
    AsyncExtractorsResource,
    ExtractorsResourceWithRawResponse,
    AsyncExtractorsResourceWithRawResponse,
    ExtractorsResourceWithStreamingResponse,
    AsyncExtractorsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_custom_raw_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from .classifiers import (
    ClassifiersResource,
    AsyncClassifiersResource,
    ClassifiersResourceWithRawResponse,
    AsyncClassifiersResourceWithRawResponse,
    ClassifiersResourceWithStreamingResponse,
    AsyncClassifiersResourceWithStreamingResponse,
)
from ..._base_client import make_request_options

__all__ = ["ContextResource", "AsyncContextResource"]


class ContextResource(SyncAPIResource):
    @cached_property
    def classifiers(self) -> ClassifiersResource:
        return ClassifiersResource(self._client)

    @cached_property
    def extractors(self) -> ExtractorsResource:
        return ExtractorsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ContextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return ContextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return ContextResourceWithStreamingResponse(self)

    def list_files(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        source_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BinaryAPIResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/x-apache-arrow-stream", **(extra_headers or {})}
        return self._get(
            "/context/files",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "source_id": source_id,
                    },
                    context_list_files_params.ContextListFilesParams,
                ),
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncContextResource(AsyncAPIResource):
    @cached_property
    def classifiers(self) -> AsyncClassifiersResource:
        return AsyncClassifiersResource(self._client)

    @cached_property
    def extractors(self) -> AsyncExtractorsResource:
        return AsyncExtractorsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncContextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncContextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncContextResourceWithStreamingResponse(self)

    async def list_files(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        source_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncBinaryAPIResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/x-apache-arrow-stream", **(extra_headers or {})}
        return await self._get(
            "/context/files",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "source_id": source_id,
                    },
                    context_list_files_params.ContextListFilesParams,
                ),
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class ContextResourceWithRawResponse:
    def __init__(self, context: ContextResource) -> None:
        self._context = context

        self.list_files = to_custom_raw_response_wrapper(
            context.list_files,
            BinaryAPIResponse,
        )

    @cached_property
    def classifiers(self) -> ClassifiersResourceWithRawResponse:
        return ClassifiersResourceWithRawResponse(self._context.classifiers)

    @cached_property
    def extractors(self) -> ExtractorsResourceWithRawResponse:
        return ExtractorsResourceWithRawResponse(self._context.extractors)


class AsyncContextResourceWithRawResponse:
    def __init__(self, context: AsyncContextResource) -> None:
        self._context = context

        self.list_files = async_to_custom_raw_response_wrapper(
            context.list_files,
            AsyncBinaryAPIResponse,
        )

    @cached_property
    def classifiers(self) -> AsyncClassifiersResourceWithRawResponse:
        return AsyncClassifiersResourceWithRawResponse(self._context.classifiers)

    @cached_property
    def extractors(self) -> AsyncExtractorsResourceWithRawResponse:
        return AsyncExtractorsResourceWithRawResponse(self._context.extractors)


class ContextResourceWithStreamingResponse:
    def __init__(self, context: ContextResource) -> None:
        self._context = context

        self.list_files = to_custom_streamed_response_wrapper(
            context.list_files,
            StreamedBinaryAPIResponse,
        )

    @cached_property
    def classifiers(self) -> ClassifiersResourceWithStreamingResponse:
        return ClassifiersResourceWithStreamingResponse(self._context.classifiers)

    @cached_property
    def extractors(self) -> ExtractorsResourceWithStreamingResponse:
        return ExtractorsResourceWithStreamingResponse(self._context.extractors)


class AsyncContextResourceWithStreamingResponse:
    def __init__(self, context: AsyncContextResource) -> None:
        self._context = context

        self.list_files = async_to_custom_streamed_response_wrapper(
            context.list_files,
            AsyncStreamedBinaryAPIResponse,
        )

    @cached_property
    def classifiers(self) -> AsyncClassifiersResourceWithStreamingResponse:
        return AsyncClassifiersResourceWithStreamingResponse(self._context.classifiers)

    @cached_property
    def extractors(self) -> AsyncExtractorsResourceWithStreamingResponse:
        return AsyncExtractorsResourceWithStreamingResponse(self._context.extractors)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["HistoryListFlowsParams"]


class HistoryListFlowsParams(TypedDict, total=False):
    max_timestamp: int

    min_timestamp: int

    registry_identifier: str



from typing import ClassVar

import quarkupy

from . import QuarkHistoryItem, QuarkRemoteDriver, inputs

TIMEOUT = 14440  # 4 hours


class ClassifierPromptQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:transformer:classifier_prompt"

    quark_input: inputs.ContextClassifierPromptInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.transformer.context_classifier_prompt.run(**self.quark_input,
                                                                                        timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class ClassifierParserQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:transformer:Classifier_parser"

    quark_input: inputs.ParseClassifierLlmInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.transformer.parse_classifier_llm.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class DocExtractQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:extractor:docling_extractor"

    quark_input: inputs.DocExtractQuarkInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.extractor.docling_extractor.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class DocChunkerQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:transformer:docling_chunker"

    quark_input: inputs.DocChunkerQuarkInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.transformer.docling_chunker.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class ExtractorPromptQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:transformer:extractor_prompt"

    quark_input: inputs.ContextExtractPromptInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.transformer.context_extract_prompt.run(**self.quark_input,
                                                                                     timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class ExtractorParserQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:transformer:extractor_parser"

    quark_input: inputs.ParseExtractorLlmInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.transformer.parse_extractor_llm.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class OpenAICompletionBaseQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:ai:openai_completion_base"

    quark_input: inputs.OpenAICompletionsInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.ai.openai_completion_base.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class OpenAIEmbeddingsQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:ai:openai_embeddings"

    quark_input: inputs.OpenAIEmbeddingsInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.ai.openai_embeddings.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class OpendalReadQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:files:opendal_list_binary"

    quark_input: inputs.OpendalInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.files.opendal.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class S3ReadCSVQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:files:s3_read_csv"

    quark_input: inputs.S3ReadCSVQuarkInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.files.s3_read_csv.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class S3ReadWholeFileQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:files:s3_read_files_binary"

    quark_input: inputs.S3ReadWholeFileQuarkInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.files.s3_read_files_binary.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class SaTSegmentQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:transformer:onnx_sat_segmentation"

    quark_input: inputs.SaTSegmentationInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.transformer.onnx_sat_segmentation.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class TextTemplateBaseQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:transformer:handlebars_base"

    quark_input: inputs.TextTemplateInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.transformer.handlebars_base.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class VectorDBIngestQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:vector:lancedb_ingest"

    quark_input: inputs.VectorDBIngestInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.vector.lancedb_ingest.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


class VectorDBSearchQuark(QuarkRemoteDriver):
    IDENTIFIER: ClassVar[str] = "quark:vector:lancedb_search"

    quark_input: inputs.VectorDBSearchInput

    async def execute(self) -> QuarkHistoryItem:
        api_client = quarkupy.AsyncClient(base_url=self.BASE_URL)
        res = await api_client.registry.quark.vector.lancedb_search.run(**self.quark_input, timeout=TIMEOUT)
        await api_client.close()

        self._history = res
        self._quark_id = res.quark_history_id
        await self.save_history()

        return self._history


quark_runner_mapping = [
    {
        "identifier": "quark:ai:openai_completion_base",
        "input": inputs.OpenAICompletionsInput,
        "constructor": OpenAICompletionBaseQuark,
    },
    {
        "identifier": "quark:ai:openai_embeddings",
        "input": inputs.OpenAIEmbeddingsInput,
        "constructor": OpenAIEmbeddingsQuark,
    },
    {
        "identifier": "quark:extractor:docling_extractor",
        "input": inputs.DocExtractQuarkInput,
        "constructor": DocExtractQuark,
    },
    {
        "identifier": "quark:files:s3_read_csv",
        "input": inputs.S3ReadCSVQuarkInput,
        "constructor": S3ReadCSVQuark
    },
    {
        "identifier": "quark:files:s3_read_files_binary",
        "input": inputs.S3ReadWholeFileQuarkInput,
        "constructor": S3ReadWholeFileQuark,
    },
    {"identifier": "quark:files:opendal_list_binary",
     "input": inputs.OpendalInput,
     "constructor": OpendalReadQuark,
     },
    {
        "identifier": "quark:transformer:classifier_prompt",
        "input": inputs.ContextClassifierPromptInput,
        "constructor": ClassifierPromptQuark,
    },
    {
        "identifier": "quark:transformer:extractor_prompt",
        "input": inputs.ContextExtractPromptInput,
        "constructor": ExtractorPromptQuark,
    },
    {
        "identifier": "quark:transformer:extractor_parser",
        "input": inputs.ParseExtractorLlmInput,
        "constructor": ExtractorParserQuark,
    },
    {
        "identifier": "quark:transformer:classifier_parser",
        "input": inputs.ParseClassifierLlmInput,
        "constructor": ClassifierParserQuark,
    },
    {
        "identifier": "quark:transformer:onnx_sat_segmentation",
        "input": inputs.SaTSegmentationInput,
        "constructor": SaTSegmentQuark,
    },
    {
        "identifier": "quark:transformer:docling_chunker",
        "input": inputs.DocChunkerQuarkInput,
        "constructor": DocChunkerQuark,
    },
    {
        "identifier": "quark:transformer:handlebars_base",
        "input": inputs.TextTemplateInput,
        "constructor": TextTemplateBaseQuark,
    },
    {
        "identifier": "quark:vector:lancedb_ingest",
        "input": inputs.VectorDBIngestInput,
        "constructor": VectorDBIngestQuark,
    },
    {
        "identifier": "quark:vector:lancedb_search",
        "input": inputs.VectorDBSearchInput,
        "constructor": VectorDBSearchQuark,
    },
]



"""Manual chaining (i.e. outside of lattices) of Quarks for RAG ingest"""

import os
import pprint
from urllib.parse import urlparse

import dotenv
import pandas as pd
from halo import Halo

import quarkupy.lib as q
import quarkupy.lib.agent as qa

# Configs
dotenv.load_dotenv()

API_KEY = os.environ.get("QUARK_API_KEY")
OPENAI_API_KEY = os.environ.get("OPENAI_API_KEY")
API_END_POINT = "https://demo.quarklabs.ai/api/quark"

if API_KEY is None or API_KEY == "":
    raise ValueError("QUARK_API_KEY environment variable not set")
if OPENAI_API_KEY is None or OPENAI_API_KEY == "":
    raise ValueError("OPENAI_API_KEY environment variable not set")

SPINNER = "dots"



async def main():
    print(q.__banner__)
    with Halo(text="💠 Getting file list...\n", spinner=SPINNER):
        agent = qa.KnowledgeAgent(
            base_url=API_END_POINT,
            openai_api_key=OPENAI_API_KEY,
        )

        try:
            answer = await agent.ask("How do I get support?")
            pprint.pprint(answer)

        except KeyboardInterrupt:
            print("Interrupted")
        except Exception as e:
            print(f"An error occurred while fetching files: {e}")
            raise e


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from quarkupy import Quarkupy, AsyncQuarkupy
from tests.utils import assert_matches_type
from quarkupy.types import (
    Source,
    SuccessResponseMessage,
    SourceListItemsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSource:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: Quarkupy) -> None:
        source = client.source.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: Quarkupy) -> None:
        response = client.source.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = response.parse()
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: Quarkupy) -> None:
        with client.source.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = response.parse()
            assert_matches_type(Source, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: Quarkupy) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.source.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: Quarkupy) -> None:
        source = client.source.update(
            config={
                "access_key_id": "access_key_id",
                "endpoint": "endpoint",
                "region": "region",
                "secret_access_key": "secret_access_key",
                "url": "url",
            },
            name="name",
        )
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: Quarkupy) -> None:
        source = client.source.update(
            config={
                "access_key_id": "access_key_id",
                "endpoint": "endpoint",
                "region": "region",
                "secret_access_key": "secret_access_key",
                "url": "url",
            },
            name="name",
            description="description",
        )
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: Quarkupy) -> None:
        response = client.source.with_raw_response.update(
            config={
                "access_key_id": "access_key_id",
                "endpoint": "endpoint",
                "region": "region",
                "secret_access_key": "secret_access_key",
                "url": "url",
            },
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = response.parse()
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: Quarkupy) -> None:
        with client.source.with_streaming_response.update(
            config={
                "access_key_id": "access_key_id",
                "endpoint": "endpoint",
                "region": "region",
                "secret_access_key": "secret_access_key",
                "url": "url",
            },
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = response.parse()
            assert_matches_type(Source, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Quarkupy) -> None:
        source = client.source.list()
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Quarkupy) -> None:
        response = client.source.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = response.parse()
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Quarkupy) -> None:
        with client.source.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = response.parse()
            assert_matches_type(SuccessResponseMessage, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_add_all_items(self, client: Quarkupy) -> None:
        source = client.source.add_all_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_add_all_items_with_all_params(self, client: Quarkupy) -> None:
        source = client.source.add_all_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            path="path",
        )
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_add_all_items(self, client: Quarkupy) -> None:
        response = client.source.with_raw_response.add_all_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = response.parse()
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_add_all_items(self, client: Quarkupy) -> None:
        with client.source.with_streaming_response.add_all_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = response.parse()
            assert_matches_type(SuccessResponseMessage, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_add_all_items(self, client: Quarkupy) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.source.with_raw_response.add_all_items(
                id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list_items(self, client: Quarkupy) -> None:
        source = client.source.list_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(SourceListItemsResponse, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_items_with_all_params(self, client: Quarkupy) -> None:
        source = client.source.list_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            path="path",
        )
        assert_matches_type(SourceListItemsResponse, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list_items(self, client: Quarkupy) -> None:
        response = client.source.with_raw_response.list_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = response.parse()
        assert_matches_type(SourceListItemsResponse, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list_items(self, client: Quarkupy) -> None:
        with client.source.with_streaming_response.list_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = response.parse()
            assert_matches_type(SourceListItemsResponse, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_list_items(self, client: Quarkupy) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.source.with_raw_response.list_items(
                id="",
            )


class TestAsyncSource:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncQuarkupy) -> None:
        source = await async_client.source.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncQuarkupy) -> None:
        response = await async_client.source.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = await response.parse()
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncQuarkupy) -> None:
        async with async_client.source.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = await response.parse()
            assert_matches_type(Source, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncQuarkupy) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.source.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncQuarkupy) -> None:
        source = await async_client.source.update(
            config={
                "access_key_id": "access_key_id",
                "endpoint": "endpoint",
                "region": "region",
                "secret_access_key": "secret_access_key",
                "url": "url",
            },
            name="name",
        )
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncQuarkupy) -> None:
        source = await async_client.source.update(
            config={
                "access_key_id": "access_key_id",
                "endpoint": "endpoint",
                "region": "region",
                "secret_access_key": "secret_access_key",
                "url": "url",
            },
            name="name",
            description="description",
        )
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncQuarkupy) -> None:
        response = await async_client.source.with_raw_response.update(
            config={
                "access_key_id": "access_key_id",
                "endpoint": "endpoint",
                "region": "region",
                "secret_access_key": "secret_access_key",
                "url": "url",
            },
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = await response.parse()
        assert_matches_type(Source, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncQuarkupy) -> None:
        async with async_client.source.with_streaming_response.update(
            config={
                "access_key_id": "access_key_id",
                "endpoint": "endpoint",
                "region": "region",
                "secret_access_key": "secret_access_key",
                "url": "url",
            },
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = await response.parse()
            assert_matches_type(Source, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncQuarkupy) -> None:
        source = await async_client.source.list()
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncQuarkupy) -> None:
        response = await async_client.source.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = await response.parse()
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncQuarkupy) -> None:
        async with async_client.source.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = await response.parse()
            assert_matches_type(SuccessResponseMessage, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_add_all_items(self, async_client: AsyncQuarkupy) -> None:
        source = await async_client.source.add_all_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_add_all_items_with_all_params(self, async_client: AsyncQuarkupy) -> None:
        source = await async_client.source.add_all_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            path="path",
        )
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_add_all_items(self, async_client: AsyncQuarkupy) -> None:
        response = await async_client.source.with_raw_response.add_all_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = await response.parse()
        assert_matches_type(SuccessResponseMessage, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_add_all_items(self, async_client: AsyncQuarkupy) -> None:
        async with async_client.source.with_streaming_response.add_all_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = await response.parse()
            assert_matches_type(SuccessResponseMessage, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_add_all_items(self, async_client: AsyncQuarkupy) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.source.with_raw_response.add_all_items(
                id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_items(self, async_client: AsyncQuarkupy) -> None:
        source = await async_client.source.list_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(SourceListItemsResponse, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_items_with_all_params(self, async_client: AsyncQuarkupy) -> None:
        source = await async_client.source.list_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            path="path",
        )
        assert_matches_type(SourceListItemsResponse, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list_items(self, async_client: AsyncQuarkupy) -> None:
        response = await async_client.source.with_raw_response.list_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        source = await response.parse()
        assert_matches_type(SourceListItemsResponse, source, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list_items(self, async_client: AsyncQuarkupy) -> None:
        async with async_client.source.with_streaming_response.list_items(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            source = await response.parse()
            assert_matches_type(SourceListItemsResponse, source, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_list_items(self, async_client: AsyncQuarkupy) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.source.with_raw_response.list_items(
                id="",
            )
